import { ComponentFixture, TestBed } from '@angular/core/testing';

import { FetusRelatedMeasurementsComponent } from './fetus-related-measurements.component';
import {Observation} from '@nspop/gm-web-facade-api';

describe('FetusRelatedMeasurementsComponent', () => {
  let component: FetusRelatedMeasurementsComponent;
  let fixture: ComponentFixture<FetusRelatedMeasurementsComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ FetusRelatedMeasurementsComponent ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(FetusRelatedMeasurementsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('displays the right value when getting single measurement', () => {
    const observation1: Observation = {
      type: 'weight',
      values: [
        {value: '50', name: '', unit: 'kg'},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [observation1];

    expect(component.getSingleMeasurementValue('weight')).toEqual('50 kg');
  });

  it('displays a placeholder if no observation found when getting single measurement', () => {
    component.observations = [];

    expect(component.getSingleMeasurementValue('weight')).toEqual('');
  });

  it('displays nothing if no value or unit found when getting single measurement', () => {
    const observation1: Observation = {
      type: 'weight',
      values: [
        {value: undefined, name: '', unit: undefined},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [observation1];

    expect(component.getSingleMeasurementValue('weight')).toEqual(' ');
  });
});
