import {AbstractControl, FormControl, FormGroup, FormGroupDirective, NgForm, ValidationErrors, ValidatorFn} from '@angular/forms';
import {ErrorStateMatcher} from '@angular/material/core';

export class ActivityModalValidator {
  static weekRangeValidator: (fromWeekName: string, toWeekName: string) => ValidatorFn =
    (fromWeekName: string, toWeekName: string) => (fg: AbstractControl): ValidationErrors | null => {
      const fromWeek: number = (fg as FormGroup).get(fromWeekName)?.value;
      const statusIsFinished: boolean = (fg as FormGroup).get('status')?.value === 'finished';
      const toWeek: number = (fg as FormGroup).get(toWeekName)?.value;
      const weekRangeInvalid = statusIsFinished ? fromWeek == null : (fromWeek == null || toWeek == null || fromWeek > toWeek);
      return weekRangeInvalid ? { weekRangeInvalid } : null;
    }
}

export class ActivityModalErrorStateMatcher implements ErrorStateMatcher {
  isErrorState(control: FormControl | null, form: FormGroupDirective | NgForm | null): boolean {
    return !!control?.parent?.invalid && control?.touched;
  }
}
