import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {Observable} from 'rxjs';
import {apiBase} from '../../utils/URL_STRING_LITERALS';
import {of} from 'rxjs/internal/observable/of';

@Injectable({
  providedIn: 'root'
})
export class ApiService {

  constructor(protected url: string,
              protected httpClient: HttpClient) {
    this.url = apiBase + url;
  }

  public create(resource: any): Observable<any> {
    return this.httpClient.post(this.url, resource);
  }

  public getAll(): Observable<any> {
    return this.httpClient.get(this.url);
  }

  public get(id: string): Observable<any> {
    return this.httpClient.get(`${this.url}/${id}`);
  }

  public update(resource: any, id: string, version?: string): Observable<any> {
    if (!resource || Object.entries(resource).length === 0) {
      return of('Empty resource, nothing to put');
    }
    return this.httpClient.put(version ? `${this.url}/${id}/${version}` : `${this.url}/${id}`, resource);
  }

  public delete(id: string): Observable<any> {
    return this.httpClient.delete(`${this.url}/${id}`);
  }

  protected extendUrlAndCall(urlExtension: string, callback: () => Observable<any>): Observable<any> {
    const oldUrl = this.url;
    this.url += urlExtension;

    const obs = callback();

    this.url = oldUrl;

    return obs;
  }
}
