import { Injectable } from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {ApiService} from './api.service';
import {Observable} from 'rxjs';
import {Note} from '@nspop/gm-web-facade-api';
import {catchError} from 'rxjs/operators';
import {of} from 'rxjs/internal/observable/of';

@Injectable({
  providedIn: 'root'
})
export class NoteService extends ApiService  {
  constructor(httpClient: HttpClient) {
    super('/note', httpClient);
  }
  public createNoteForActivity(activityId: string, resource: Note): Observable<any> {
    if (!resource || Object.entries(resource).length === 0) {
      return of('Empty resource, nothing to post');
    }
    return this.httpClient.post(this.url + '/' + activityId, resource);
  }
  public update(resource: any, id: string, version?: string): Observable<any> {
    if (!resource || Object.entries(resource).length === 0) {
      return of('Empty resource, nothing to post');
    }
    return super.update(resource, id, version);
  }

  public createOrUpdateNote(activityId: string | undefined, note: Note, measurementDate: Date | undefined): Observable<any> {
    if (!note || note.content.length <= 0 || !activityId) {
      return of(true);
    }
    if (note.documentUniqueId) {
      note.date = measurementDate;
      return this.update(note, note.documentUniqueId);
    } else {
      return this.createNoteForActivity(
        activityId,
        {content: note.content, date: measurementDate}
      );
    }
  }
}
