import { Pipe, PipeTransform } from '@angular/core';
import {formatDistanceToNowStrict} from 'date-fns';
import {da} from 'date-fns/locale';

@Pipe({
  name: 'calculateAge'
})
export class CalculateAgePipe implements PipeTransform {
  private static getFullYearFromCpr(cpr: string): number {
    if (cpr.length !== 10) {
      return -1;
    }
    const seventhDigit = parseInt(cpr[6], 10);
    let fullYear = '';
    const year = cpr.substring(4, 6);
    const yearAsInt = parseInt(year, 10);
    if (seventhDigit <= 3) {
      fullYear = '19' + year;
    }else if (seventhDigit === 4 || seventhDigit === 9) {
      if (yearAsInt > 36) {
        fullYear = '19' + year;
      }else {
        fullYear = '20' + year;
      }
    }else if (seventhDigit <= 8) {
      if (yearAsInt <= 36) {
        fullYear = '20' + year;
      }else if (yearAsInt >= 58){
        fullYear = '18' + year;
      }
    }
    if (fullYear.length === 0){
      return -1;
    } else {
      return parseInt(fullYear, 10);
    }
  }

  public transform(value: string, ...args: unknown[]): unknown {
    return this.calculateAge(value);
  }

  public calculateAge(cpr: string): string {
    cpr = cpr.replace('-', '');
    const birthDate = new Date(
      CalculateAgePipe.getFullYearFromCpr(cpr),
      parseInt(cpr.substring(2, 4), 10),
      parseInt(cpr.substring(0, 2), 10)
    );
    return formatDistanceToNowStrict(
      birthDate,
      {unit: 'year', roundingMethod: 'floor', locale: da}
    );
  }
}
