import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../modals/modal/modal.component';
import {NgForm} from '@angular/forms';
import {FormComponent} from '../confirm-navigation-if-dirty.framework';
import {FeedbackService} from '../services/feedback.service';
import {finalize} from 'rxjs/operators';

@Component({
  selector: 'app-feedback-modal',
  templateUrl: './feedback-modal.component.html',
  styleUrls: ['./feedback-modal.component.scss']
})
export class FeedbackModalComponent extends FormComponent implements OnChanges{
  public isSaving = false;
  public score?: number;
  public feedback = '';

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;
  @Input() public isOpen = false;
  @Input() public title = 'denne side';
  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();

  constructor(private feedbackService: FeedbackService) { super(); }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.isOpen) {
      if (this.isOpen) {
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
  }

  public handleClose(): void {
    this.modalComponent?.setVisible(false);
    this.score = undefined;
    this.feedback = '';
    this.form?.form.markAsPristine();
    this.modalClosed.emit();
  }

  publicFeedbackChange(): void {
    const smileySelector = document.getElementById('comment');
    smileySelector?.classList.remove('ng-invalid');
  }

  public doSave(): void {
    if (this.feedback) {
      if (!this.isSaving) {
        this.isSaving = true;
        this.feedbackService.sendFeedback({rating: this.score, comment: this.feedback}).pipe(
          finalize(() => this.isSaving = false)
        ).subscribe(() => {
          this.handleClose();
        });
      }
    } else {
      const smileySelector = document.getElementById('comment');
      smileySelector?.scrollIntoView({behavior: 'smooth', block: 'center'});
      smileySelector?.classList.add('ng-invalid');
    }
  }

}
