import {ComponentFixture, TestBed} from '@angular/core/testing';

import {SmileyComponent, SmileySelectComponent} from './smiley-select.component';
import {By} from '@angular/platform-browser';
import {MatIconTestingModule} from '@angular/material/icon/testing';

describe('SmileySelectComponent', () => {
  let component: SmileySelectComponent;
  let fixture: ComponentFixture<SmileySelectComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [MatIconTestingModule],
      declarations: [ SmileySelectComponent ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SmileySelectComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
    expect(component.options).toEqual([]);
    expect(component.selectedText).toEqual('');
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    component.writeValue(2);
    expect(component.value).toEqual(2);
  });

  it('only registers changes if model has structural changes', () => {
    const onChange = jasmine.createSpy('onChange');
    component.writeValue(1);
    component.registerOnChange(onChange);
    component.writeValue(1);
    expect(onChange).not.toHaveBeenCalled();
    component.makeDirtyable();
    fixture.detectChanges();
    component.writeValue(2);
    expect(onChange).toHaveBeenCalled();
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    component.registerOnChange(onChange);
    component.writeValue(2);
    expect(onChange).not.toHaveBeenCalled();
    component.makeDirtyable();
    fixture.detectChanges();
    component.writeValue(3);
    expect(onChange).toHaveBeenCalledWith(3);
  });

  it('becomes dirtyable when clicked', () => {
    component.smileys = {toArray: () => [{value: 1, label: '1', icon: '1'}, {value: 2, label: '2', icon: '2'}]} as any;
    fixture.detectChanges();
    expect(component.options.length).toBe(2);
    fixture.debugElement.query(By.css('.buttons')).query(By.css('button')).triggerEventHandler('click', {});
    fixture.detectChanges();
    expect((component as any).dirtyAble).toBeTrue();
  });

  it('marks smiley as checked when clicked', () => {
    const smiley1 = {value: 1, label: '1', icon: '1'} as SmileyComponent;
    const smiley2 = {value: 2, label: '2', icon: '2'} as SmileyComponent;
    component.smileys = {toArray: () => [smiley1, smiley2]} as any;
    fixture.detectChanges();
    component.handleClick(smiley1);
    expect(smiley1.checked).toBeTrue();
    expect(smiley2.checked).toBeFalse();
  });

  it('unmarks checked smiley when another one is clicked', () => {
    const smiley1 = {value: 1, label: '1', icon: '1'} as SmileyComponent;
    const smiley2 = {value: 2, label: '2', icon: '2'} as SmileyComponent;
    component.smileys = {toArray: () => [smiley1, smiley2]} as any;
    fixture.detectChanges();
    component.handleClick(smiley1);
    expect(smiley1.checked).toBeTrue();
    expect(smiley2.checked).toBeFalse();
    component.handleClick(smiley2);
    expect(smiley1.checked).toBeFalse();
    expect(smiley2.checked).toBeTrue();
  });

  it('updates slected text when smiley is slected', () => {
    const smiley1 = {value: 1, label: '1', icon: '1'} as SmileyComponent;
    const smiley2 = {value: 2, label: '2', icon: '2'} as SmileyComponent;
    component.smileys = {toArray: () => [smiley1, smiley2]} as any;
    fixture.detectChanges();
    component.handleClick(smiley1);
    expect(component.selectedText).toBe('1');
  });

});
