import {Component, OnInit} from '@angular/core';
import {PdfService} from '../pdf.service';
import {DialogService} from '../../modals/dialog/dialog.service';
import {DialogComponent} from '../../modals/dialog/dialog.component';
import {GeneratePdfDialogComponent} from '../generate-pdf-dialog/generate-pdf-dialog.component';
import {Subscription} from 'rxjs';
import {of} from 'rxjs/internal/observable/of';
import {DataCardService} from '../../api/data-card.service';

enum pdfStatus {
  IDLE,
  GENERATING,
  SUCCEEDED,
  CANCELLED,
  FAILED
}

enum toastStatus {
  HIDDEN,
  DISPLAY,
  HIDE,
}

@Component({
  selector: 'app-generate-pdf-button',
  templateUrl: './generate-pdf-button.component.html',
  styleUrls: ['./generate-pdf-button.component.scss']
})
export class GeneratePdfButtonComponent implements OnInit{

  public isFirstDownload: boolean = true;

  public pdfStatus: pdfStatus = pdfStatus.IDLE;
  public pdfStatusEnum = pdfStatus;

  public toastStatus: toastStatus = toastStatus.HIDDEN;
  public toastStatusEnum = toastStatus;
  private toastDisplayTimeMs = 4000;

  private pdfSubscription: Subscription = of().subscribe();

  constructor(private pdfService: PdfService,
              private dialogService: DialogService,
              private dataCardService: DataCardService
  ) { }

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe(() => {
      this.pdfStatus = pdfStatus.IDLE;
      this.isFirstDownload = true;
    });
  }

  generatePdf(): void {
    this.pdfStatus = pdfStatus.GENERATING;
    this.pdfSubscription = this.pdfService.generatePdf().subscribe((blob: Blob) => {
      this.pdfStatus = pdfStatus.SUCCEEDED;
      this.isFirstDownload = false;
      this.displayToast();
      this.pdfService.savePdf(blob);
    }, (error => {
      this.pdfStatus = pdfStatus.FAILED;
      this.displayToast();
      console.error(error);
    }));
  }

  openDialog(): void {
    this.dialogService
      .show(DialogComponent, {
        data: {
          header: 'Download graviditetsforløb',
          component: GeneratePdfDialogComponent,
          noButtonText: 'Fortryd',
          okButtonText: 'Download'
        },
        width: '440'})
      .subscribe((result) => {
        if (result) {
          this.generatePdf();
        }
      });
  }

  cancelGeneratingPdf(): void {
    this.pdfStatus = pdfStatus.CANCELLED;
    this.displayToast();
    this.pdfSubscription.unsubscribe();
  }

  displayToast(): void {
    this.toastStatus = toastStatus.DISPLAY;
    setTimeout(() => this.hideToast(), this.toastDisplayTimeMs);
  }

  hideToast(): void {
    this.toastStatus = toastStatus.HIDE;
    setTimeout(() => this.toastStatus = toastStatus.HIDDEN, 1000);
  }
}
