import {AfterViewInit, Directive, ElementRef, Input} from '@angular/core';
import {ActivatedRoute, ParamMap, Params, Router} from '@angular/router';
import {timer} from 'rxjs';

@Directive({
  selector: '[appHighlightOnNavigation]'
})
export class HighlightOnNavigationDirective implements AfterViewInit{
  @Input('appHighlightOnNavigation') id: string|undefined;

  constructor(private elementRef: ElementRef, private activeRoute: ActivatedRoute, private router: Router) {}

  ngAfterViewInit(): void {
    this.activeRoute.queryParamMap.subscribe((params: ParamMap) => {
      if (params.has('highlight')) {
        const highlightId = params.get('highlight');
        if (highlightId === this.id) {
          this.removeHighlightParam().then(() => {
            setTimeout(() => { // Push scrolling to the back of the event queue to make sure we have something to scroll to
              this.elementRef.nativeElement.scrollIntoView({behavior: 'smooth', block: 'center'});
              this.elementRef.nativeElement.classList.add('highlighted');
              const blinker = timer(0, 800).subscribe(() => {
                this.elementRef.nativeElement.classList.remove('hover');
                setTimeout(() => this.elementRef.nativeElement.classList.add('hover'), 300 );
              });
              setTimeout(() => {
                blinker.unsubscribe();
                this.elementRef.nativeElement.classList.add('hover');
              },  1600);
            });
          });
        }
      }
    });
  }

  removeHighlightParam(): Promise<void> {
    const queryParams: Params = { highlight: undefined};
    return this.router.navigate(
      [],
      {
        relativeTo: this.activeRoute,
        queryParams,
        queryParamsHandling: 'merge'
      }).then(() => undefined);
  }

}
