// https://github.com/text-mask/text-mask/blob/master/componentDocumentation.md#readme
import {TextMaskConfig} from 'angular2-text-mask';

export type MaskType = 'cpr' | 'phone' | 'zip';

export const CPR_MASK = [/\d/, /\d/, /\d/, /\d/, /\d/, /\d/, '-', /[a-zA-Z0-9]/, /[a-zA-Z0-9]/, /[a-zA-Z0-9]/, /[a-zA-Z0-9]/];
export const PHONE_MASK = [/\d/, /\d/, ' ', /\d/, /\d/, ' ', /\d/, /\d/, ' ', /\d/, /\d/];
export const ZIP_MASK = [/\d/, /\d/, /\d/, /\d/];

export function getTextMask(
  maskType: MaskType | undefined,
  guide?: boolean,
  placeholderChar?: string,
  pipe?: (conformedValue: string, config: TextMaskConfig) => false | string | object,
  keepCharPositions?: boolean,
  showMask?: boolean,
  hidePlaceholder: boolean = true
): TextMaskConfig {
  let mask: false | (string | RegExp)[] = false;

  switch (maskType) {
    case 'cpr':
      mask = CPR_MASK;
      break;
    case 'phone':
      mask = PHONE_MASK;
      break;
    case 'zip':
      mask = ZIP_MASK;
      break;
  }

  return {
    mask,
    guide,
    placeholderChar: hidePlaceholder ? '\u2000' : undefined,
    pipe,
    keepCharPositions,
    showMask
  };
}
