import {AfterViewInit, Component, ElementRef, forwardRef, Input, ViewChild} from '@angular/core';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import deepEqual from 'deep-equal';
import {fromEvent} from 'rxjs';

export const APP_BLOOD_PRESSURE_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BloodPressureInputComponent),
  multi: true
};

@Component({
  selector: 'app-blood-pressure-input',
  templateUrl: './blood-pressure-input.component.html',
  styleUrls: ['./blood-pressure-input.component.scss'],
  providers: [APP_BLOOD_PRESSURE_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ],
})
export class BloodPressureInputComponent implements AfterViewInit, ControlValueAccessor {
  @ViewChild('systolicInput', {static: true}) systolicInput: ElementRef<HTMLInputElement>|undefined;
  @ViewChild('diastolicInput', {static: true}) diastolicInput: ElementRef<HTMLInputElement>|undefined;

  @Input() public disabled = false;

  public isSystolicFocused = false;
  public isDiastolicFocused = false;

  private innerModel: any = {};
  private dirtyAble = false;
  private onChangeCallback: (newValue?: any) => any = () => {};
  private onTouchedCallback: () => any = () => {};
  constructor() { }

  ngAfterViewInit(): void {
    if (this.systolicInput) {
      fromEvent<KeyboardEvent>(this.systolicInput.nativeElement, 'keydown').subscribe((event) => {
        if (event.key === '/' || event.key === '-' || event.key === ':') {
          event.preventDefault();
          event.returnValue = false;
          this.handleDiastolicFocus();
        }
      });
    }
  }

  get systolic(): number|undefined {
    return this.innerModel.systolic;
  }

  set systolic(value: number|undefined) {
    if (value !== this.innerModel?.systolic) {
      this.innerModel.systolic = value;
      this.handleNgModelBookKeeping();
    }
  }

  get diastolic(): number|undefined {
    return this.innerModel?.diastolic;
  }

  set diastolic(value: number|undefined) {
    if (value !== this.innerModel?.diastolic) {
      this.innerModel.diastolic = value;
      this.handleNgModelBookKeeping();
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: any): void {
    if (!deepEqual(this.innerModel, newValue)) {
      this.innerModel = newValue || {};
      this.handleNgModelBookKeeping();
    }
  }

  handleSystolicFocus(): void {
    if (this.isSystolicFocused) { return; }

    this.dirtyAble = true;
    this.isSystolicFocused = true;
    this.systolicInput?.nativeElement?.focus?.();
  }

  handleDiastolicFocus(): void {
    if (this.isDiastolicFocused) { return; }

    this.isDiastolicFocused = true;
    this.diastolicInput?.nativeElement?.focus?.();
  }

  private handleNgModelBookKeeping(): void {
    if (!this.innerModel?.systolic && !this.innerModel?.diastolic) {
      this.innerModel = {};
    }
    if (this.dirtyAble) {
      this.onChangeCallback(deepEqual(this.innerModel, {}) ? undefined : this.innerModel);
    }
  }

}

