import {ComponentFixture, TestBed} from '@angular/core/testing';

import {UltrasoundModalSectionComponent} from './ultrasound-modal-section.component';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {AppModule} from '../../../app.module';
import {FormsModule, NgForm} from '@angular/forms';
import {DataCardWrapper} from '../../../model-wrappers/dataCard.wrapper';

describe('UltrasoundModalSectionComponent', () => {
  let component: UltrasoundModalSectionComponent;
  let fixture: ComponentFixture<UltrasoundModalSectionComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [UltrasoundModalSectionComponent],
      imports: [MatIconTestingModule, AppModule, FormsModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(UltrasoundModalSectionComponent);
    component = fixture.componentInstance;
    component.observationDataUltrasound = {
      fetuses: [{}, {}]
    } as any;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('adds fetuses if number of fetuses in data-card increases', () => {
    fixture.detectChanges();
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(3);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(3);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('does not remove fetuses if data-card has fetus-count 0', () => {
    fixture.detectChanges();
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(0);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(2);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('does not remove fetuses if data-card has no fetus-count', () => {
    fixture.detectChanges();
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(undefined);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(2);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  it('removes fetuses if number if fetuses in data-card descreases', () => {
    fixture.detectChanges();
    // @ts-ignore
    component.observationDataUltrasound.fetuses[0].ulWeight = 1500;
    changeFetusCount(1);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(1);
    // @ts-ignore
    expect(component.observationDataUltrasound?.fetuses[0].ulWeight).toBe(1500);
  });

  describe('when dataCard is undefined', () => {
    beforeEach(() => {
      component.dataCard = undefined;
    });

    it('does not set dataCard values', () => {
      fixture.detectChanges();
      component.handleTrimesterScanUpdate('trimester1');
      component.expectedDateOfBirthFromUltrasound = new Date();
      component.downsProbability = 'high';
      component.fetusCount = 2;
      component.multiplicityRisk = 'false';
      component.multiplicityRiskComment = 'comment';

      expect(component.dataCard).toBe(undefined);
    });
  });

  describe('when dataCard is defined', () => {
    beforeEach(() => {
      component.dataCard = new DataCardWrapper({} as any);
    });

    describe('and multiplicityRisk is already set', () => {
      const riskValue = 'false';
      beforeEach(() => {
        component.dataCard = new DataCardWrapper({} as any);
        component.multiplicityRisk = riskValue;
      });

      it('sets multiplicityRiskComment without overwriting multiplicityRisk', () => {
        fixture.detectChanges();
        expect(component.dataCard?.riskAssessment2Section).toBeTruthy();

        const value = 'comment';
        component.multiplicityRiskComment = value;

        expect(component.dataCard?.riskAssessment2Section?.dirty).toBe(true);
        expect(component.dataCard?.riskAssessment2Section?.multiplicityRiskAssessment?.comment).toEqual(value);
        expect(component.dataCard?.riskAssessment2Section?.multiplicityRiskAssessment?.risk).toEqual(riskValue);
      });
    });

    it('handles trimester scan updates', () => {
      fixture.detectChanges();
      // The function creates a date itself, so we have to freeze date and time
      const fixedDate = new Date();
      jasmine.clock().install();
      jasmine.clock().mockDate(fixedDate);

      expect(component.dataCard?.riskAssessment3Section).toBeFalsy();

      component.handleTrimesterScanUpdate('trimester1');

      expect(component.dataCard?.riskAssessment3Section).toBeTruthy();
      expect(component.dataCard?.riskAssessment3Section?.dirty).toBe(true);
      expect(component.dataCard?.riskAssessment3Section?.firstTrimesterUltrasoundScanOfFetusCompleted).toEqual(fixedDate);

      expect(component.dataCard?.riskAssessment4Section).toBeFalsy();

      component.handleTrimesterScanUpdate('trimester2');

      expect(component.dataCard?.riskAssessment4Section).toBeTruthy();
      expect(component.dataCard?.riskAssessment4Section?.dirty).toBe(true);
      expect(component.dataCard?.riskAssessment4Section?.secondTrimesterUltrasoundScanOfFetusCompleted).toEqual(fixedDate);

      // Unfreeze
      jasmine.clock().uninstall();
    });

    it('sets expectedDateOfDelivery', () => {
      fixture.detectChanges();
      expect(component.dataCard?.ultrasoundSection).toBeFalsy();

      const value = new Date();
      component.expectedDateOfBirthFromUltrasound = value;

      expect(component.dataCard?.ultrasoundSection).toBeTruthy();
      expect(component.dataCard?.ultrasoundSection?.dirty).toBe(true);
      expect(component.dataCard?.ultrasoundSection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan).toEqual(value);
    });

    it('sets downsProbability', () => {
      fixture.detectChanges();
      expect(component.dataCard?.riskAssessment3Section).toBeFalsy();

      const value = 'low';
      component.downsProbability = value;

      expect(component.dataCard?.riskAssessment3Section).toBeTruthy();
      expect(component.dataCard?.riskAssessment3Section?.dirty).toBe(true);
      expect(component.dataCard?.riskAssessment3Section?.downsScreeningProbabilityAssessment).toEqual(value);
    });

    it('sets fetusCount', () => {
      fixture.detectChanges();
      expect(component.dataCard?.multiplePregnancySection).toBeFalsy();

      const value = 1;
      component.fetusCount = value;

      expect(component.dataCard?.multiplePregnancySection).toBeTruthy();
      expect(component.dataCard?.multiplePregnancySection?.dirty).toBe(true);
      expect(component.dataCard?.multiplePregnancySection?.numberOfFetuses).toEqual(value);
    });

    it('sets multiplicityRisk', () => {
      fixture.detectChanges();
      expect(component.dataCard?.riskAssessment2Section).toBeFalsy();


      const value = 'false';
      component.multiplicityRisk = value;

      expect(component.dataCard?.riskAssessment2Section).toBeTruthy();
      expect(component.dataCard?.riskAssessment2Section?.dirty).toBe(true);
      expect(component.dataCard?.riskAssessment2Section?.multiplicityRiskAssessment?.risk).toEqual(value);
    });

    it('sets multiplicityRiskComment', () => {
      fixture.detectChanges();
      expect(component.dataCard?.riskAssessment2Section).toBeFalsy();

      const value = 'comment';
      component.multiplicityRiskComment = value;

      expect(component.dataCard?.riskAssessment2Section).toBeTruthy();
      expect(component.dataCard?.riskAssessment2Section?.dirty).toBe(true);
      expect(component.dataCard?.riskAssessment2Section?.multiplicityRiskAssessment?.comment).toEqual(value);
    });
  });

  function changeFetusCount(count?: number): void {
    component.dataCard = new DataCardWrapper({
      expectedDateOfDeliverySection: {},
      prenatalRiskAssessmentSection: {
        finalRiskAssessment: {
          multiplicityRiskAssessment: {}
        },
      },
      multiplePregnancySection: {numberOfFetuses: count},
    } as any);
    component.ngOnChanges({dataCard: {}} as any);
    fixture.detectChanges();
  }
});
