import {Component, forwardRef, Input, OnChanges, SimpleChanges} from '@angular/core';
import {RiskAssessment3, SaveUltrasoundObservationsRequest, YesNoEnum} from '@nspop/gm-web-facade-api';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCardWrapper} from '../../../model-wrappers/dataCard.wrapper';
import deepEqual from 'deep-equal';
import {copy} from '../../../../utils/copy';
import DownsScreeningProbabilityAssessmentEnum = RiskAssessment3.DownsScreeningProbabilityAssessmentEnum;
import {Data} from '@angular/router';

export const APP_ULTRASOUND_MODAL_SECTION_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => UltrasoundModalSectionComponent),
  multi: true
};

type ScanningType = 'trimester1' | 'trimester2' | 'other';

@Component({
  selector: 'app-ultrasound-modal-section',
  templateUrl: './ultrasound-modal-section.component.html',
  styleUrls: ['./ultrasound-modal-section.component.scss'],
  providers: [APP_ULTRASOUND_MODAL_SECTION_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class UltrasoundModalSectionComponent implements OnChanges, ControlValueAccessor {
  @Input()  dataCard: DataCardWrapper|undefined;
  @Input() public readOnlyMode: boolean = false;

  public trimesterScan?: ScanningType;
  public observationDataUltrasound: SaveUltrasoundObservationsRequest | undefined;

  private innerExpectedDateOfBirthFromUltrasound: Date | undefined;
  private innerDownsProbability: DownsScreeningProbabilityAssessmentEnum | undefined;
  private innerFetusCount: number | undefined;
  private innerMultiplicityRisk: YesNoEnum | undefined;
  private innerMultiplicityRiskComment: string | undefined;

  private dirtyAble = false;

  public onChangeCallback: (newValue: SaveUltrasoundObservationsRequest | undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() { }

  get value(): SaveUltrasoundObservationsRequest | undefined {
    return this.observationDataUltrasound;
  }

  set value(value: SaveUltrasoundObservationsRequest | undefined) {
    if (!deepEqual(value, this.observationDataUltrasound)) {
      this.observationDataUltrasound = value;
      if (this.dirtyAble) {

        this.onChangeCallback(value);
      }
      if (!this.dirtyAble && this.observationDataUltrasound) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
      if (!this.observationDataUltrasound) { // reset dirty when data is reset.
        this.dirtyAble = false;
      }
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.dataCard) {
      this.handleFetusCountUpdate(this.dataCard?.fetusCount || 0);
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: SaveUltrasoundObservationsRequest): void {
    this.value = newValue;
  }

  public handleFetusCountUpdate(fetusCount: number): void {
    if (this.observationDataUltrasound && this.observationDataUltrasound.fetuses) {
      if (!fetusCount || fetusCount === this.observationDataUltrasound.fetuses.length) {
        return; // If the user is claiming to have 0 feti something is wrong... We'll hold off on deleting stuff
      }
      if (fetusCount < this.observationDataUltrasound.fetuses.length) {
        this.observationDataUltrasound.fetuses = this.observationDataUltrasound.fetuses.slice(0, fetusCount);
      }
      if (fetusCount > this.observationDataUltrasound.fetuses.length) {
        for (let neededFeti = fetusCount - this.observationDataUltrasound.fetuses.length; neededFeti > 0; neededFeti--) {
          this.observationDataUltrasound.fetuses.push(
            {id: `Foster ${this.observationDataUltrasound.fetuses.length + 1}`}
          );
        }
      }
      this.observationDataUltrasound = copy(this.observationDataUltrasound); // Trigger onChange if model has changed
    }
  }

  public handleTrimesterScanUpdate(value: ScanningType): void {
    if (!this.dataCard) {
      return;
    }

    switch (value) {
      case 'trimester1':
        this.dataCard.riskAssessment3Section = this.dataCard.riskAssessment3Section || {};
        this.dataCard.riskAssessment3Section.dirty = true;
        this.dataCard.riskAssessment3Section.firstTrimesterUltrasoundScanOfFetusCompleted = new Date();
        break;
      case 'trimester2':
        this.dataCard.riskAssessment4Section = this.dataCard.riskAssessment4Section || {};
        this.dataCard.riskAssessment4Section.dirty = true;
        this.dataCard.riskAssessment4Section.secondTrimesterUltrasoundScanOfFetusCompleted = new Date();
        break;
      case 'other':
      default:
        break;
    }
    this.onChangeCallback(this.value); // Mark form as dirty for updates to this field
  }

  get expectedDateOfBirthFromUltrasound(): Date | undefined {
    return this.innerExpectedDateOfBirthFromUltrasound;
  }

  set expectedDateOfBirthFromUltrasound(value: Date | undefined) {
    this.innerExpectedDateOfBirthFromUltrasound = value;
    if (!this.dataCard) {
      return;
    }
    this.dataCard.ultrasoundSection = this.dataCard.ultrasoundSection || {};
    this.dataCard.ultrasoundSection.dirty = true;
    this.dataCard.expectedDateOfBirthFromUltrasound = value;
    console.log('I was called');
  }

  get downsProbability(): DownsScreeningProbabilityAssessmentEnum | undefined {
    return this.innerDownsProbability;
  }

  set downsProbability(value: DownsScreeningProbabilityAssessmentEnum | undefined) {
    this.innerDownsProbability = value;

    if (!this.dataCard) {
      return;
    }
    this.dataCard.riskAssessment3Section = this.dataCard.riskAssessment3Section || {};
    this.dataCard.riskAssessment3Section.dirty = true;
    this.dataCard.downsProbability = value;
    this.onChangeCallback(this.value);
  }

  get fetusCount(): number | undefined {
    return this.innerFetusCount;
  }

  set fetusCount(value: number | undefined) {
    this.innerFetusCount = value;

    if (!this.dataCard ) {
      return;
    }
    this.dataCard.multiplePregnancySection = this.dataCard.multiplePregnancySection || {};
    this.dataCard.multiplePregnancySection.dirty = true;
    this.dataCard.multiplePregnancySection.numberOfFetuses = value;
    this.onChangeCallback(this.value);
  }

  get multiplicityRisk(): YesNoEnum | undefined {
    return this.innerMultiplicityRisk;
  }

  set multiplicityRisk(value: YesNoEnum | undefined) {
    this.innerMultiplicityRisk = value;

    if (!this.dataCard) {
      return;
    }
    this.dataCard.riskAssessment2Section = this.dataCard.riskAssessment2Section || {};
    this.dataCard.riskAssessment2Section.dirty = true;
    this.dataCard.multiplicityRisk = value;
    this.onChangeCallback(this.value);
  }
  get multiplicityRiskComment(): string | undefined {
    return this.innerMultiplicityRiskComment;
  }

  set multiplicityRiskComment(value: string | undefined) {
    this.innerMultiplicityRiskComment = value;

    if (!this.dataCard) {
      return;
    }
    this.dataCard.riskAssessment2Section = this.dataCard.riskAssessment2Section || {};
    this.dataCard.riskAssessment2Section.dirty = true;
    this.dataCard.multiplicityRiskComment = value;
    this.onChangeCallback(this.value);
  }
}
