import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';
import {ClinicalExamsLineChartComponent} from './clinical-exams-line-chart.component';
import {By} from '@angular/platform-browser';
import {of} from 'rxjs/internal/observable/of';
import {Observable} from 'rxjs';
import {ObservationTable} from '@nspop/gm-web-facade-api';

describe('ClinicalExamsLineChartComponent', () => {
  let component: ClinicalExamsLineChartComponent;
  let fixture: ComponentFixture<ClinicalExamsLineChartComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ ClinicalExamsLineChartComponent ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ClinicalExamsLineChartComponent);
    component = fixture.componentInstance;
    component.data = of({
      mother: {rows: [
          {weeksIntoPregnancy: 10, pubismeasurement: {value: 40, unit: 'mm'}},
          {weeksIntoPregnancy: 15, pubismeasurement: {value: 80, unit: 'mm'}},
          {weeksIntoPregnancy: 20, pubismeasurement: {value: 150, unit: 'mm'}},
          {weeksIntoPregnancy: 30, pubismeasurement: {value: 300, unit: 'mm'}},
        ]}
    }) as Observable<ObservationTable>;
    fixture.detectChanges();
  });

  it('should create a chart', () => {
    component.renderChart(600);
    expect(component.svg?.attr('width')).toBeGreaterThanOrEqual(600);
  });

  it('draws a chart', fakeAsync(() => {
    component.ngAfterViewInit();
    fixture.detectChanges();
    tick();
    const points = fixture.debugElement.queryAll(By.css('svg circle'));
    expect(points.length).toBe(4);
    expect(Math.floor(+points[0].nativeNode.attributes.getNamedItem('cx').value)).toEqual(95);
    expect(Math.floor(+points[0].nativeNode.attributes.getNamedItem('cy').value)).toEqual(328);

    expect(Math.floor(+points[3].nativeNode.attributes.getNamedItem('cx').value)).toEqual(413);
    expect(Math.floor(+points[3].nativeNode.attributes.getNamedItem('cy').value)).toEqual(120);
  }));
});
