import { ComponentFixture, TestBed } from '@angular/core/testing';

import { SummaryCardComponent } from './summary-card.component';
import {AppModule} from '../../../app.module';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {ActivitiesService} from '../../../activities/activities.service';

describe('SummaryCardComponent', () => {
  let component: SummaryCardComponent;
  let fixture: ComponentFixture<SummaryCardComponent>;
  let activitiesServiceStub: any;
  beforeEach(async () => {
    activitiesServiceStub = {
      emitOpenNoteModalEvent: { subscribe: () => {}}
    } as any;
    await TestBed.configureTestingModule({
      declarations: [ SummaryCardComponent ],
      imports: [AppModule, MatIconTestingModule],
      providers: [
        {provide: ActivitiesService, useValue: activitiesServiceStub}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SummaryCardComponent);
    component = fixture.componentInstance;
    component.activity = {} as any;
    component.showActivityModal = {
      isOpen: false
    };
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should change the modal properties when activity is shown', () => {
    const activityId = '1234-abcd';
    expect(component.showActivityModal?.isOpen).toBeFalsy();
    component.showActivity(activityId);
    expect(component.showActivityModal?.isOpen).toBeTrue();
    expect(component.showActivityModal?.activityId).toEqual(activityId);
  });

  it('should open the edit note modal when the editNote button is pressed', () => {
    const spy = spyOn(activitiesServiceStub, 'emitOpenNoteModalEvent');
    component.editNote();
    expect(spy).toHaveBeenCalledTimes(1);
  });
});
