import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {MeasurementsTabComponent} from './measurements-tab.component';
import {TabsComponent} from '../../tabs/tabs.component';
import {MatTabsModule} from '@angular/material/tabs';
import {Component, forwardRef} from '@angular/core';
import {Observable, of, ReplaySubject, Subject} from 'rxjs';
import {ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {RouterTestingModule} from '@angular/router/testing';

import {DataCard} from '@nspop/gm-web-facade-api';
import anything = jasmine.anything;
import stringMatching = jasmine.stringMatching;
import {DataCardService} from '../../api/data-card.service';
import {ActivitiesService} from '../../activities/activities.service';

@Component({selector: 'app-measurements-midwife', template: ''})
class MidwifeStubComponent {
}
const APP_MOCK_ULTRASOUND_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => UltrasoundStubComponent),
  multi: true
};

const APP_MOCK_SCREENINGS_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => ScreeningsStubComponent),
  multi: true
};

@Component({selector: 'app-measurements-ultrasound', template: '', providers: [APP_MOCK_ULTRASOUND_CONTROL_VALUE_ACCESSOR]})
class UltrasoundStubComponent implements ControlValueAccessor {
  registerOnChange(fn: any): void {}
  registerOnTouched(fn: any): void {}
  writeValue(newValue: DataCard): void {}
}
@Component({selector: 'app-measurements-summary', template: ''})
class SummaryStubComponent {
}
@Component({selector: 'app-measurements-screenings', template: '', providers: [APP_MOCK_SCREENINGS_CONTROL_VALUE_ACCESSOR]})
class ScreeningsStubComponent implements ControlValueAccessor {
  registerOnChange(fn: any): void {}
  registerOnTouched(fn: any): void {}
  writeValue(newValue: DataCard): void {}
}

describe('MeasurementsTabComponent', () => {
  let component: MeasurementsTabComponent;
  let fixture: ComponentFixture<MeasurementsTabComponent>;
  let dataCardServiceStub: DataCardService;
  let activitiesServiceStub: ActivitiesService;

  beforeEach(async () => {
    dataCardServiceStub = {
      update: () => of(),
      selectedDataCard$: of({documentUniqueId: 'identifier', version: '2'})
    } as any;
    activitiesServiceStub = {
      openNoteModalEvent$: { subscribe: () => {}}
    } as any;

    await TestBed.configureTestingModule({
      imports: [MatTabsModule, FormsModule, RouterTestingModule],
      declarations: [MeasurementsTabComponent,
        TabsComponent,
        MidwifeStubComponent,
        UltrasoundStubComponent,
        SummaryStubComponent,
        ScreeningsStubComponent
      ],
      providers: [
        { provide: ActivitiesService, useValue:  activitiesServiceStub},
        { provide: DataCardService, useValue:  dataCardServiceStub},
        { provide: NgForm, useValue: new NgForm([], []) },
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(MeasurementsTabComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('extracts relevant Ids from dataCard so we can save later', fakeAsync(() => {
    const spy = spyOn(dataCardServiceStub, 'update').and.returnValue(of());
    component.ngOnInit();
    component.form = {valid: true} as any;
    tick();
    component.save();
    expect(spy).toHaveBeenCalledWith(anything(), stringMatching('identifier'), stringMatching('2'));
  }));

  it('does not save if form is not valid', fakeAsync(() => {
    const spy = spyOn(dataCardServiceStub, 'update').and.returnValue(of());
    component.ngOnInit();
    component.form = {valid: false} as any;
    tick();
    component.save();
    expect(spy).not.toHaveBeenCalled();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new MeasurementsTabComponent({selectedDataCard$: of()} as any, activitiesServiceStub as any);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));
});
