import {Component, EventEmitter, Input, OnChanges, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {Activity, HealthCarePerformer, Observation, ObservationType} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../../api/care-plan.service';

@Component({
  selector: 'app-show-related-activity-modal',
  templateUrl: './show-related-activity-modal.component.html',
  styleUrls: ['./show-related-activity-modal.component.scss']
})
export class ShowRelatedActivityModalComponent implements OnChanges {

  constructor(private carePlanService: CarePlanService) {
  }

  public activity: Activity | undefined;
  public feti: Set<string | undefined> = new Set();
  public readonly ObservationType = ObservationType;

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @Input() public activityId: string | undefined;
  @Input() public observationType: 'regular' | 'ultrasound' | undefined;
  @Input() public isOpen = false;
  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.activityId && this.activityId) {
      const activity = this.carePlanService.getActivityById(this.activityId);
      if (!activity) {
        throw new Error(`Could not find activity with id '${this.activityId}' in the care-plan`);
      }
      this.activity = activity;
      this.feti = new Set(activity.observations?.map(x => x.fetus).filter(x => !!x));
      this.feti = this.feti.size === 0 ? new Set(['']) : this.feti;
    }
    if (changes.isOpen) {
      if (this.isOpen) {
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
  }

  public handleClose(): void {
    this.modalComponent?.setVisible(false);
    this.modalClosed.emit();
  }

  public getObservation(type: ObservationType, fetus?: string): string {
    const relevantObservation: Observation | undefined = this.activity
      ?.observations
      ?.filter(x => x.type === type && (!fetus || x.fetus === fetus))?.[0];
    if (relevantObservation) {
      const value = type === 'blood_pressure'
        ? relevantObservation.values[0].value + '/' + relevantObservation.values[1].value
        : relevantObservation.values[0].value;
      const unit = relevantObservation.values[0].unit ? ' ' + relevantObservation.values[0].unit : '';
      return value + unit;
    }
    return '-';
  }

  public getMeasuredBy(observations: Array<Observation>): HealthCarePerformer | undefined {
    return observations.find(observation => observation.measuredBy != null)?.measuredBy;
  }

  getMeasurerAddress(measurer: HealthCarePerformer): string {
    const address = measurer.healthCareOrganizationAdress?.city ?
      ', ' + measurer.healthCareOrganizationAdress.city : '';
    return measurer.healthCareOrganization?.name + address;
  }
}
