import {Component, forwardRef, OnInit, ViewChild} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {ModalComponent} from '../../modals/modal/modal.component';
import {Activity} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../../api/care-plan.service';

export const TIE_TO_ACTIVITY_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => TieToActivityModalComponent),
  multi: true
};

@Component({
  selector: 'app-tie-to-activity-modal',
  templateUrl: './tie-to-activity-modal.component.html',
  styleUrls: ['./tie-to-activity-modal.component.scss'],
  providers: [TIE_TO_ACTIVITY_CONTROL_VALUE_ACCESSOR]
})
export class TieToActivityModalComponent implements OnInit, ControlValueAccessor {
  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  public innerModel?: Activity = undefined;
  public currentValue: Activity | undefined;
  public displayText = '';
  public disabled = false;
  public hover = false;
  public activities: Activity[] = [];

  private onChangeCallback: (newValue?: Activity) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor(private carePlanService: CarePlanService) { }

  public ngOnInit(): void {
    this.carePlanService.activities$.subscribe((activities) => {
      this.activities = activities.filter(activity => activity.status !== 'cancelled');
      if (this.currentValue) {
        this.innerModel = this.activities.find(x => x.id === this.currentValue);
        this.displayText = this.getDisplayText();
      }
    });
  }

  get value(): Activity|undefined{
    return this.innerModel;
  }

  set value(value: Activity|undefined) {
    if (value?.id !== this.innerModel?.id) {
      this.innerModel = this.activities.find(x => x.id === this.currentValue?.id);
      this.onChangeCallback(value);
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: Activity): void {
    this.currentValue = newValue;
    this.value = newValue;
    this.displayText = this.getDisplayText();
  }

  public handleClose(): void {
    this.modalComponent?.setVisible(false);
    this.value = this.currentValue;
  }

  public open(): void {
    if (!this.disabled) {
      this.modalComponent?.setVisible(true);
    }
  }

  public pickActivity(): void {
    this.currentValue = this.innerModel;
    this.displayText = this.getDisplayText();
    this.onChangeCallback(this.innerModel);
    this.handleClose();
  }

  private getDisplayText(): string {
    return this.activities.find(x => x.id === this.currentValue?.id)?.title || '';
  }

}
