import {Component, ContentChildren, EventEmitter, Input, OnInit, Output, QueryList} from '@angular/core';
import {animate, style, transition, trigger} from '@angular/animations';
import {NgForm} from '@angular/forms';
import {DirtyGuard, FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {Observable} from 'rxjs';

const timings = '250ms';

@Component({
  selector: 'app-modal',
  templateUrl: './modal.component.html',
  styleUrls: ['./modal.component.scss'],
  animations: [
    trigger('slideInOut', [
      transition(':enter', [
        style({transform: 'translateX(100%)'}),
        animate(timings + ' ease-in-out', style({transform: 'translateX(0%)'}))
      ]),
      transition(':leave', [
        animate(timings + ' ease-in-out', style({transform: 'translateX(100%)'}))
      ])
    ]),
    trigger(
      'fadeInOut',
      [
        transition(
          ':enter',
          [
            style({ opacity: 0 }),
            animate(timings + ' ease-out',
              style({opacity: 0.3 }))
          ]
        ),
        transition(
          ':leave',
          [
            animate(timings + ' ease-out',
            style({ opacity: 0}))
          ]
        )
      ]
    )
  ]
})
export class ModalComponent extends FormComponent {
  @Input() public header = '';
  @Input() public subHeader = '';
  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  @ContentChildren(NgForm, {descendants: true}) forms: QueryList<NgForm>|undefined;

  public isModalVisible = false;
  private showingDirtyCheckDialog = false;
  constructor(private dirtyGuard: DirtyGuard) { super(); }

  get form(): NgForm | undefined {
    return this.forms?.first;
  }

  public setVisible(isVisible: boolean): void {
    const emitClosedEvent = this.isModalVisible && !isVisible;
    if (emitClosedEvent && this.form) {
      if (!this.showingDirtyCheckDialog) {
        const canClose = this.dirtyGuard.canDeactivate(this);
        this.showingDirtyCheckDialog = true;
        if (typeof canClose === 'boolean') {
          if (canClose) {
            this.updateVisibility(isVisible, emitClosedEvent);
          }
          this.showingDirtyCheckDialog = false;
        } else if ((canClose as Promise<boolean>).then != null) {
          (canClose as Promise<boolean>).then((continueClosing) => {
            if (continueClosing) {
              this.updateVisibility(isVisible, emitClosedEvent);
            }
            this.showingDirtyCheckDialog = false;
          });
        } else if ((canClose as Observable<boolean>).subscribe != null) {
          (canClose as Observable<boolean>).subscribe((continueClosing) => {
            if (continueClosing) {
              this.updateVisibility(isVisible, emitClosedEvent);
            }
            this.showingDirtyCheckDialog = false;
          });
        } else {
          this.updateVisibility(isVisible, emitClosedEvent);
          this.showingDirtyCheckDialog = false;
        }
      }
    } else {
      this.updateVisibility(isVisible, emitClosedEvent);
    }
  }

  private updateVisibility(isVisible: boolean, emitClosedEvent: boolean): void {
    this.isModalVisible = isVisible;
    if (emitClosedEvent) {
      this.modalClosed.emit();
    }
  }
}

@Component({
  selector: 'app-modal-controls',
  template: '<ng-content></ng-content>'
})
export class ModalControlsComponent {}
