import {NgModule} from '@angular/core';
import {Route, RouterModule, Routes} from '@angular/router';
import {MeasurementsTabComponent} from '../measurements/measurements-tab/measurements-tab.component';
import {ActivitiesComponent} from '../activities/activities.component';
import {PregnancyComponent} from '../pregnancy/pregnancy.component';
import {PatientInformationComponent} from '../patient-information/patient-information.component';
import {TriageOverviewComponent} from '../triage-overview/triage-overview.component';
import {PrivatelyMarkedComponent} from '../privately-marked/privately-marked.component';
import {NegatePrivatelyMarkedGuard} from '../privately-marked/negate-privately-marked.guard';
import {SomethingWentWrongComponent} from '../something-went-wrong/something-went-wrong.component';
import {appUrls} from '../../utils/URL_STRING_LITERALS';
import {ComponentType} from '@angular/cdk/portal';
import {PrivatelyMarkedGuard} from '../privately-marked/privately-marked.guard';
import {DirtyGuard} from '../confirm-navigation-if-dirty.framework';
import {LogoutComponent} from '../logout/logout.component';
import {NoPatientChosenComponent} from '../no-patient-chosen/no-patient-chosen.component';
import {CprDoesNotExistComponent} from '../cpr-does-not-exist/cpr-does-not-exist.component';
import {NoActivePathwayComponent} from '../no-active-pathway/no-active-pathway.component';
import {PatientInContextGuard} from '../patient-in-context.guard';
import {NegatePatientInContextGuard} from '../negate-patient-in-context.guard';
import {ActivePathwayGuard} from '../no-active-pathway/active-pathway.guard';
import {NoActivePathwayGuard} from '../no-active-pathway/no-active-pathway.guard';

/**
 * The application routing parameters
 * NOTE: Order matters - matches first hit
 */
export const appRoutes: Routes = [
  route(appUrls.logout, LogoutComponent),
  openDataCardRoute(appUrls.triageOverview, TriageOverviewComponent),
  openDataCardRoute(appUrls.activities, ActivitiesComponent),
  openDataCardRoute(appUrls.measurements, MeasurementsTabComponent),
  openDataCardRoute(appUrls.pregnancy, PregnancyComponent),
  openDataCardRoute(appUrls.patientInformation, PatientInformationComponent),
  route(appUrls.privatelyMarked, PrivatelyMarkedComponent, [NegatePrivatelyMarkedGuard]),
  route(appUrls.somethingWentWrong, SomethingWentWrongComponent),
  route(appUrls.noPatientChosen, NoPatientChosenComponent, [NegatePatientInContextGuard]),
  route(appUrls.cprDoesNotExist, CprDoesNotExistComponent, [NegatePatientInContextGuard]),
  route(appUrls.noActivePathway, NoActivePathwayComponent, [PrivatelyMarkedGuard, PatientInContextGuard, NoActivePathwayGuard]),
  { path: appUrls.base, redirectTo: appUrls.activities}, // Default/wildcard redirect
];

function route(path: string, component?: ComponentType<any>, canActivate?: ComponentType<any>[]): Route {
  return {
    path,
    component,
    canActivate,
    canDeactivate: [DirtyGuard]
  };
}

function openDataCardRoute(path: string, component?: ComponentType<any>, canActivate?: ComponentType<any>[]): Route {
  return route(path, component, [PrivatelyMarkedGuard, PatientInContextGuard, ActivePathwayGuard, ...(canActivate || [])]);
}

@NgModule({
  imports: [RouterModule.forRoot(appRoutes, {useHash: true})],
  exports: [RouterModule]
})

export class RoutingModule { }
