import {Injectable} from '@angular/core';
import {ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot} from '@angular/router';
import {PatientContextService} from '../services/patient-context.service';
import {appUrls} from '../../utils/URL_STRING_LITERALS';

@Injectable({
  providedIn: 'root'
})
export class ActivePathwayGuard implements CanActivate {
  constructor(private router: Router,
              private patientContextService: PatientContextService) {
  }

  hasActivePathway(): boolean {
    return !!this.patientContextService.getCurrentEpisodeOfCare();
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): boolean {
    const hasActivePathway = this.hasActivePathway();

    if (!hasActivePathway && !state.url.includes(appUrls.noActivePathway)) {
      this.router.navigate([appUrls.noActivePathway]);
    }

    return hasActivePathway;
  }
}
