import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {NoActivePathwayComponent} from './no-active-pathway.component';
import {Observable, of} from 'rxjs';
import {DataCardService} from '../api/data-card.service';
import SpyObj = jasmine.SpyObj;
import createSpyObj = jasmine.createSpyObj;
import {MatDialogModule} from '@angular/material/dialog';
import {PatientContextService} from '../services/patient-context.service';
import {createPatientContextSpyObject} from '../services/patient-context.service.spec';
import {DataCard} from '@nspop/gm-web-facade-api';
import {getPropertyAsSpy} from '../../utils/spy';

describe('NoActivePathwayComponent', () => {
  let component: NoActivePathwayComponent;
  let fixture: ComponentFixture<NoActivePathwayComponent>;
  let dataCardServiceStub: SpyObj<DataCardService>;
  let patientContextServiceStub: SpyObj<PatientContextService>;

  beforeEach(async () => {
    patientContextServiceStub = createPatientContextSpyObject();

    dataCardServiceStub = createSpyObj<DataCardService>(
      {
      create: of({} as any),
      refresh: undefined,
      },
      {
        allDataCards$: of([])
      }
      );

    await TestBed.configureTestingModule({
      declarations: [ NoActivePathwayComponent ],
      imports: [MatDialogModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: PatientContextService, useValue: patientContextServiceStub},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(NoActivePathwayComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when there are no closed dataCards', () => {
    beforeEach(() => {
      getPropertyAsSpy(dataCardServiceStub, 'allDataCards$').and.returnValue(
        of(
          [
            {documentStatus: 'ACTIVE'},
          ]) as any as Observable<DataCard[]>
      );
    });

    it('sets property isThereAnyEarlierPregnancies to false', () => {
      expect(component.isThereAnyEarlierPregnancies).toBe(false);
      fixture.detectChanges();
      expect(component.isThereAnyEarlierPregnancies).toBe(false);
    });
  });

  describe('when there are closed dataCards', () => {
    beforeEach(() => {
      getPropertyAsSpy(dataCardServiceStub, 'allDataCards$').and.returnValue(
        of(
        [
          {documentStatus: 'ACTIVE'},
          {documentStatus: 'CLOSED'}
        ]) as any as Observable<DataCard[]>
      );
    });

    it('sets property isThereAnyEarlierPregnancies to false', () => {
      expect(component.isThereAnyEarlierPregnancies).toBe(false);
      fixture.detectChanges();
      expect(component.isThereAnyEarlierPregnancies).toBe(true);
    });
  });

  it('triggers data reload after creating a new episode of care', fakeAsync(() => {
    fixture.detectChanges();
    component.createPathway();
    tick();
    expect(dataCardServiceStub.refresh).toHaveBeenCalled();
  }));
});
