import {Component, OnInit} from '@angular/core';
import {DataCardService} from '../api/data-card.service';
import {SwitchEncounterComponent} from '../switch-encounter/switch-encounter.component';
import {DataCard} from '@nspop/gm-web-facade-api';
import {MatDialog} from '@angular/material/dialog';
import {PatientContextService} from '../services/patient-context.service';

@Component({
  selector: 'app-no-active-pathway',
  templateUrl: './no-active-pathway.component.html',
  styleUrls: ['./no-active-pathway.component.scss']
})
export class NoActivePathwayComponent implements OnInit {

  public isThereAnyEarlierPregnancies: boolean = false;

  constructor(
    private dataCardService: DataCardService,
    private dialog: MatDialog,
    private patientContextService: PatientContextService
  ) {
  }

  public ngOnInit(): void {
    this.dataCardService.allDataCards$.subscribe((dataCards) => {
      this.isThereAnyEarlierPregnancies = dataCards.some((dataCard) => dataCard.documentStatus === 'CLOSED');
    });
  }

  public createPathway(): void {
    this.dataCardService.create().subscribe(() => {
      this.dataCardService.refresh();
    });
  }

  public openSwitchEncounterDialog(): void {
    const dialogRef = this.dialog.open(SwitchEncounterComponent);
    dialogRef.afterClosed().subscribe((result: DataCard) => {
      if (result) {
        this.patientContextService.setEpisodeOfCare(result.episodeOfCareIdentifier);
      }
    });
  }
}
