import {TestBed} from '@angular/core/testing';

import {NoActivePathwayGuard} from './no-active-pathway.guard';
import {ActivatedRouteSnapshot, Router, RouterStateSnapshot} from '@angular/router';
import {ActivePathwayGuard} from './active-pathway.guard';
import SpyObj = jasmine.SpyObj;
import createSpyObj = jasmine.createSpyObj;

describe('NoActivePathwayGuard', () => {
  let guard: NoActivePathwayGuard;
  let activePathwayGuardMock: SpyObj<ActivePathwayGuard>;
  let routerMock: SpyObj<Router>;
  let activatedRouteStub: ActivatedRouteSnapshot;

  beforeEach(async () => {
    routerMock = createSpyObj<Router>('Router', ['navigate']);

    activatedRouteStub = {} as any;

    activePathwayGuardMock = createSpyObj<ActivePathwayGuard>('ActivePathwayGuard', ['hasActivePathway', 'canActivate']);

    await TestBed.configureTestingModule({
      imports: [],
      providers: [
        {provide: Router, useValue: routerMock},
        {provide: ActivePathwayGuard, useValue: activePathwayGuardMock},
      ],
    }).compileComponents();
    guard = TestBed.inject(NoActivePathwayGuard);
  });

  function fakeRouterState(url: string): RouterStateSnapshot {
    return {
      url,
    } as RouterStateSnapshot;
  }

  it('should be created', () => {
    expect(guard).toBeTruthy();
  });

  describe('when having a patient in context', () => {
    beforeEach(() => {
      activePathwayGuardMock.hasActivePathway.and.returnValue(true);
    });

    it('does not activate', () => {
      const canActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(canActivate).toBe(!activePathwayGuardMock.hasActivePathway());
      expect(canActivate).toBe(false);
    });
  });

  describe('when NOT having a patient in context', () => {
    beforeEach(() => {
      activePathwayGuardMock.hasActivePathway.and.returnValue(false);
    });

    it('activates', () => {
      const canActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(canActivate).toBe(!activePathwayGuardMock.hasActivePathway());
      expect(canActivate).toBe(true);
    });
  });
});
