import {Injectable} from '@angular/core';
import {ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot} from '@angular/router';
import {PatientContextService} from './services/patient-context.service';
import {appUrls} from '../utils/URL_STRING_LITERALS';

@Injectable({
  providedIn: 'root'
})
export class PatientInContextGuard implements CanActivate {
  constructor(private router: Router,
              private patientContextService: PatientContextService) {
  }

  hasPatientInContext(): boolean {
    return !!this.patientContextService.getCurrentPatientExists()
    && !!this.patientContextService.getCurrentCpr();
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): boolean {
    const hasPatientInContext = this.hasPatientInContext();

    const errorRoutes: string[] = [appUrls.noPatientChosen, appUrls.somethingWentWrong, appUrls.cprDoesNotExist];
    const isOnErrorPageMap: boolean[] = errorRoutes.map(r => state.url.includes(r));
    const isOnAnyErrorPage: boolean = isOnErrorPageMap.reduce(((prev, curr) => prev ? prev : curr));

    if (!hasPatientInContext && !isOnAnyErrorPage) {
      this.router.navigate([appUrls.noPatientChosen]);
    }

    return hasPatientInContext;
  }
}
