import { ComponentFixture, TestBed } from '@angular/core/testing';

import { PatientInformationInputsComponent } from './patient-information-inputs.component';
import {DataCard, PersonalDataSectionNextOfKin} from '@nspop/gm-web-facade-api';
import {JsonTransformerUtil} from '../../../utils/json-transformer.util';
import RelationEnum = PersonalDataSectionNextOfKin.RelationEnum;

describe('PatientInformationInputsComponent', () => {
  let component: PatientInformationInputsComponent;
  let fixture: ComponentFixture<PatientInformationInputsComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ PatientInformationInputsComponent ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PatientInformationInputsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('only registers changes if model has structural changes', () => {
    const dataCard1 = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    const dataCard2 = JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard;
    // tslint:disable-next-line:no-non-null-assertion It's not null because it's a copy of dataCard1 which has the property set...
    dataCard2.previousAbortionsSection!.abortions!.push({
      year: undefined,
      type: 'induced',
      gestationLength: {
        value: 6,
        unit: 'w'
      },
      indication: 'whatever'
    });
    const onChange = jasmine.createSpy('onChange');
    component.writeValue(dataCard1 );
    component.registerOnChange(onChange);
    component.writeValue(JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(0);
    component.writeValue(dataCard2 as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(1);
  });

  it('translates relations to next of kin', () => {
    Object.values(RelationEnum).forEach((value: string) => {
      expect(component.displayRelation(value)).not.toBe('?');
      expect(component.displayRelation(value)).not.toBe(value);
    });
  });

  it('translates unknown relations to next to "?"', () => {
    expect(component.displayRelation('an_unreasonable_code')).toBe('?');
    // The next two cases should never happen, but we don't want to crash if they do
    expect(component.displayRelation(null as unknown as string)).toBe('?');
    expect(component.displayRelation(undefined as unknown as string)).toBe('?');
  });


});
