import {Component, forwardRef} from '@angular/core';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {DataCard} from '@nspop/gm-web-facade-api';
import deepEqual from 'deep-equal';

export const APP_PATIENT_INFOMRATION_INPUTS_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => PatientInformationInputsComponent),
  multi: true
};

@Component({
  selector: 'app-patient-information-inputs',
  templateUrl: './patient-information-inputs.component.html',
  styleUrls: ['./patient-information-inputs.component.scss'],
  providers: [APP_PATIENT_INFOMRATION_INPUTS_CONTROL_VALUE_ACCESSOR],
  viewProviders: [{provide: ControlContainer, useExisting: NgForm}]
})
export class PatientInformationInputsComponent implements ControlValueAccessor {
  public activePractitionerTab = 1;

  private innerModel: DataCardWrapper | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard|undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() {
  }

  get value(): DataCardWrapper | undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper | undefined) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
    if (this.value.healthInsuranceSection?.insuranceGroup === '2') {
      this.activePractitionerTab = 2;
    }
  }

  public displayRelation(relation: string): string {
    switch (relation) {
      case 'uspec_pårørende':
        return 'Ukendt';
      case 'ingen_relationer':
        return 'Ingen relation';
      case 'barn':
        return 'Barn';
      case 'mor':
        return 'Mor';
      case 'far':
        return 'Far';
      case 'forælder':
        return 'Forælder';
      case 'søskende':
        return 'Søskende';
      case 'ægtefælle':
        return 'Ægtefælle';
      case 'barnebarn':
        return 'Barnebarn';
      case 'svigerbarn':
        return 'Svigerbarn';
      case 'øvrig_familie':
        return 'Øvrig familie';
      case 'nabo':
        return 'Nabo';
      case 'samboende':
        return 'Samboende';
      case 'registreret_partner':
        return 'Registreret partner';
      case 'anden':
        return 'Anden';
      case 'søn':
        return 'Søn';
      case 'datter':
        return 'Datter';
      default:
        return '?';
    }
  }

}
