import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';

import {PatientInformationComponent} from './patient-information.component';
import {of} from 'rxjs';
import {Component, Directive, Input} from '@angular/core';
import {FormsModule} from '@angular/forms';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {CardComponent} from '../card/card.component';
import {NamePipe} from './name.pipe';
import {OptionalDatePipe} from './optional-date.pipe';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {MatMenuModule} from '@angular/material/menu';
import {MatIconModule} from '@angular/material/icon';
import {MaskableInputComponent} from '../maskable-input/maskable-input.component';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {AppModule} from '../app.module';
import {DataCardService} from '../api/data-card.service';
import {createDataCardServiceSpy} from '../api/data-card.service.spec';
import {getPropertyAsSpy} from '../../utils/spy';
import SpyObj = jasmine.SpyObj;

@Component({
  selector: 'app-radio-group',
  template: '<ng-content></ng-content>'
})
class MockGroupWrapperComponent {
}

@Component({
  selector: 'app-general-information-bar',
  template: '<p>information-bar</p>'
})
class MockGeneralInformationBarComponent {
}

@Component({
  selector: 'app-semi-fixed-header',
  template: '<ng-content></ng-content>'
})
class MockSemiFixedHeaderComponent {
  // tslint:disable-next-line:no-input-rename
  @Input('fixed-at') fixedAt = 0;
}

// tslint:disable-next-line:directive-selector
@Directive({selector: 'mask'})
class MaskMockDirective {
}

describe('PatientInformationComponent', () => {
  let component: PatientInformationComponent;
  let fixture: ComponentFixture<PatientInformationComponent>;

  let dataCardServiceStub: SpyObj<DataCardService>;

  beforeEach(async () => {
    dataCardServiceStub = createDataCardServiceSpy();

    dataCardServiceStub.update.and.returnValue(of({} as any));
    dataCardServiceStub.refresh.and.returnValue(of({} as any).toPromise());
    getPropertyAsSpy(dataCardServiceStub, 'isDataCardClosed').and.returnValue(of(false));
    getPropertyAsSpy(dataCardServiceStub, 'selectedDataCard$').and.returnValue(of() as any);

    await TestBed.configureTestingModule({
      declarations: [
        PatientInformationComponent,
        MockGroupWrapperComponent,
        MockGeneralInformationBarComponent,
        NamePipe,
        OptionalDatePipe,
        MaskableInputComponent,
        MockSemiFixedHeaderComponent,
        MaskMockDirective,
        CardComponent
      ],
      imports: [
        AngularMaterialModule,
        AppModule,
        FormsModule,
        NoopAnimationsModule,
        MatCheckboxModule,
        MatMenuModule,
        MatIconModule
      ],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PatientInformationComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('disables save button while saving', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.isSubmitting).toBeFalse();
    component.save();
    expect(component.isSubmitting).toBeTrue();
    flush();
    expect(component.isSubmitting).toBeFalse();
  }));


  it('handles that active datacard could be undefined', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.dataCard).toBeUndefined();
  }));
});
