import {AfterViewChecked, AfterViewInit, Component, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {Subscription} from 'rxjs';
import {DataCard} from '@nspop/gm-web-facade-api';
import {NgForm} from '@angular/forms';
import {FormComponent} from '../confirm-navigation-if-dirty.framework';
import {copy} from '../../utils/copy';
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-patient-information',
  templateUrl: './patient-information.component.html',
  styleUrls: ['./patient-information.component.scss']
})
export class PatientInformationComponent extends FormComponent implements OnInit, OnDestroy {
  @ViewChild('form', {static: true}) form: NgForm|undefined;
  public dataCard: DataCard | undefined;
  private documentUniqueId = '';
  private version = '';
  private subscription: Subscription | undefined;
  public isSubmitting = false;
  public activePractitionerTab = 1;

  constructor(private dataCardService: DataCardService) {
    super();
  }

  ngOnInit(): void {
    this.subscription = this.dataCardService.selectedDataCard$
      .subscribe((dataCard?: DataCard) => {
        if (dataCard) {
          this.documentUniqueId = dataCard.documentUniqueId || '';
          this.version = dataCard.version || '';
          this.dataCard = copy(dataCard);
        }
      });
  }

  ngOnDestroy(): void {
    this.subscription?.unsubscribe();
  }

  public save(): void {
    this.isSubmitting = true;
    this.dataCardService.update(this.dataCard as DataCard, this.documentUniqueId, this.version)
      .subscribe(() => {
        this.dataCardService.refresh().then(() => {
          this.isSubmitting = false;
          this.submitDone();
        });
      }, () => this.isSubmitting = false);
  }
}
