import {Component, forwardRef} from '@angular/core';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCard} from '@nspop/gm-web-facade-api';
import deepEqual from 'deep-equal';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {addDays} from 'date-fns';


export const APP_MEDICAL_HISTORY_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MedicalHistoryComponent),
  multi: true
};


@Component({
  selector: 'app-medical-history',
  templateUrl: './medical-history.component.html',
  styleUrls: ['./medical-history.component.scss'],
  providers: [APP_MEDICAL_HISTORY_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class MedicalHistoryComponent implements ControlValueAccessor  {
  public yes = true;
  public no = false;

  private heightThreshold: number = 99;
  private weightThreshold: number = 9;
  private innerBmi: number | undefined;

  private innerModel: DataCardWrapper | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard|undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor() { }

  get value(): DataCardWrapper|undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper|undefined) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }

      this.updateInnerBmi();
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
  }

  public updateInnerBmi(): void {
    const height = this.value?.bmiHeight;
    const weight = this.value?.bmiWeight;
    let bmi: number | undefined;
    if (height && weight && height > this.heightThreshold && weight > this.weightThreshold) {
      bmi = Math.round((weight / Math.pow(height / 100, 2)) * 100) / 100;
    }

    if (this.value) {
      this.value.bmi = bmi;
    }
    this.innerBmi = bmi;
  }

  get bmi(): number | undefined {
    return this.innerBmi;
  }

  public lastPeriodChanged($event: any): void {
    if (this.value) {
      this.value.naegelsDateOfBirth = addDays($event, 280);
    }
  }
}
