import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';
import {PregnancyComponent} from './pregnancy.component';
import {AppModule} from '../app.module';
import {of} from 'rxjs';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {FormsModule} from '@angular/forms';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {DataCardService} from '../api/data-card.service';
import {createDataCardServiceSpy} from '../api/data-card.service.spec';
import {getPropertyAsSpy} from '../../utils/spy';
import SpyObj = jasmine.SpyObj;

describe('PregnancyComponent', () => {
  let component: PregnancyComponent;
  let fixture: ComponentFixture<PregnancyComponent>;
  let dataCardServiceStub: SpyObj<DataCardService>;

  beforeEach(async () => {

    dataCardServiceStub = createDataCardServiceSpy();

    dataCardServiceStub.update.and.returnValue(of({} as any));
    dataCardServiceStub.refresh.and.returnValue(of({} as any).toPromise());
    getPropertyAsSpy(dataCardServiceStub, 'isDataCardClosed').and.returnValue(of(false));
    getPropertyAsSpy(dataCardServiceStub, 'selectedDataCard$').and.returnValue(of() as any);

    await TestBed.configureTestingModule({
      declarations: [PregnancyComponent],
      imports: [AppModule, AngularMaterialModule, FormsModule, MatIconTestingModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PregnancyComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('disables save button while saving', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.isSubmitting).toBeFalse();
    component.save();
    expect(component.isSubmitting).toBeTrue();
    flush();
    expect(component.isSubmitting).toBeFalse();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    fixture.detectChanges();
    flush();
    expect(component.dataCard).toBeUndefined();
  }));
});
