import {Injectable} from '@angular/core';
import {WindowService} from './window.service';
import {UserAuthorizationType} from './user-context.service';
import {SorItem} from './sor.service';
import deepEqual from 'deep-equal';

export interface SessionStorageUserContext {
  commonName?: string;
  role?: UserAuthorizationType;
  organizationSor?: SorItem;
  userIdentifier?: number;
}

export interface SessionStoragePatientContext {
  cpr?: string;
  xBreakTheGlassReason?: string;
}

type SessionStorageType = 'user_context' | 'patient_context';

@Injectable({
  providedIn: 'root'
})
export class SessionStorageService {


  constructor(private windowService: WindowService) {
  }

  // Generic functions

  private getSessionStorageTypeName(type: SessionStorageType): string {
    return `dgaf_${type}`;
  }

  private getSessionStorageItem(item: string): string | undefined {
    return this.windowService
      .nativeWindow()
      .sessionStorage
      .getItem(item);
  }

  private setSessionStorageItem(item: string, value: any): void {
    this.windowService
      .nativeWindow()
      .sessionStorage
      .setItem(item, value);
  }

  private deleteSessionStorageItem(item: string): void {
    this.windowService
      .nativeWindow()
      .sessionStorage
      .removeItem(item);
  }

  public clear(): void {
    this.windowService
      .nativeWindow()
      .sessionStorage
      .clear();
  }

  // User Context Storage

  public getUserContext(): SessionStorageUserContext {
    const context = this.getSessionStorageItem(this.getSessionStorageTypeName('user_context'));
    if (context) {
      return JSON.parse(context);
    }
    return {};
  }

  public setUserContext(context?: SessionStorageUserContext): void {
    this.setSessionStorageItem(this.getSessionStorageTypeName('user_context'), JSON.stringify(context));
    if (deepEqual(this.getUserContext(), {})) {
      this.deleteUserContext();
    }
  }

  public setUserIdentifier(userIdentifier?: number): void {
    this.setUserContext({...this.getUserContext(), userIdentifier});
  }

  public setUserContextCommonName(commonName?: string): void {
    this.setUserContext({...this.getUserContext(), commonName});
  }

  public setUserContextRole(role?: UserAuthorizationType): void {
    this.setUserContext({...this.getUserContext(), role});
  }

  public setUserContextOrganization(organizationSor?: SorItem): void {
    this.setUserContext({...this.getUserContext(), organizationSor});
  }

  public deleteUserContext(): void {
    this.deleteSessionStorageItem(this.getSessionStorageTypeName('user_context'));
  }

  // Patient Context Storage

  public getPatientContext(): SessionStoragePatientContext {
    const context = this.getSessionStorageItem(this.getSessionStorageTypeName('patient_context'));
    if (context) {
      return JSON.parse(context);
    }
    return {};
  }

  public setPatientContext(context?: SessionStoragePatientContext): void {
    this.setSessionStorageItem(this.getSessionStorageTypeName('patient_context'), JSON.stringify(context));
    if (deepEqual(this.getPatientContext(), {})) {
      this.deletePatientContext();
    }
  }

  public setPatientContextCpr(cpr?: string): void {
    this.setPatientContext({...this.getPatientContext(), cpr});
  }

  public setPatientContextXBreakTheGlassReason(xBreakTheGlassReason?: string): void {
    this.setPatientContext({...this.getPatientContext(), xBreakTheGlassReason});
  }

  public deletePatientContext(): void {
    this.deleteSessionStorageItem(this.getSessionStorageTypeName('patient_context'));
  }

}
