import {Inject, LOCALE_ID, Pipe, PipeTransform} from '@angular/core';
import {formatDate} from '@angular/common';
import {DataCard} from '@nspop/gm-web-facade-api';

@Pipe({
  name: 'cardStatus'
})
export class DatacardStatusPipe implements PipeTransform {

  constructor(@Inject(LOCALE_ID) private locale: string) {
  }

  transform(dataCard: DataCard, ...args: unknown[]): unknown {
    switch (dataCard.documentStatus) {
      case 'CLOSED':
        const closedInterpretation = `Afsluttet`;
        if (dataCard.closeDateTime) {
          return `${closedInterpretation} (d. ${formatDate(dataCard.closeDateTime, 'dd.MM.yyyy', this.locale)})`;
        }
        return closedInterpretation;
      case 'ACTIVE':
        // tslint:disable-next-line:max-line-length
        const dateOfDelivery: Date | undefined = dataCard.ultrasoundSection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan ||
          dataCard.expectedDateOfDeliverySection?.expectedDateOfDeliveryFromLastPeriod?.expectedDateOfDelivery;
        const activeInterpretation = `Igangværende`;
        if (dateOfDelivery) {
          return `${activeInterpretation} (termin d. ${formatDate(dateOfDelivery, 'dd.MM.yyyy', this.locale)})`;
        } else {
          return activeInterpretation;
        }
    }
  }
}
