import {ActivityPlannedTime, DataCard} from '@nspop/gm-web-facade-api';
import moment, {Moment} from 'moment';

export function calculateFromAndToWeek(dataCard: DataCard | undefined, fromWeek: number, toWeek: number): ActivityPlannedTime {
  const pregnancyStartDate = dataCard?.expectedDateOfDeliverySection?.offsetDate;

  if (!pregnancyStartDate || !fromWeek || !toWeek) {
    return {
      from: undefined,
      to: undefined,
    } as ActivityPlannedTime;
  }

  const fromMoment: Moment = moment(pregnancyStartDate).add(fromWeek, 'weeks');
  const toMoment: Moment = moment(pregnancyStartDate).add(toWeek, 'weeks');

  return {
    from: fromMoment?.toDate(),
    to: toMoment?.toDate(),
  } as ActivityPlannedTime;
}

export function calculateWeekNumberFromPregnancyStart(dataCard: DataCard | undefined, date: Date | undefined): number | undefined {
  const pregnancyStartMoment = dataCard?.expectedDateOfDeliverySection?.offsetDate
    ? moment(dataCard?.expectedDateOfDeliverySection?.offsetDate)
    : undefined;

  const dateMoment = date ? moment(date) : undefined;

  if (!pregnancyStartMoment || !dateMoment) { return; }

  return dateMoment.diff(pregnancyStartMoment, 'week');
}
